# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1beta1InferenceServicesConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'explainers': 'V1beta1ExplainersConfig',
        'resource': 'V1beta1ResourceConfig',
        'service_annotation_disallowed_list': 'list[str]',
        'service_label_disallowed_list': 'list[str]'
    }

    attribute_map = {
        'explainers': 'explainers',
        'resource': 'resource',
        'service_annotation_disallowed_list': 'serviceAnnotationDisallowedList',
        'service_label_disallowed_list': 'serviceLabelDisallowedList'
    }

    def __init__(self, explainers=None, resource=None, service_annotation_disallowed_list=None, service_label_disallowed_list=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1InferenceServicesConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._explainers = None
        self._resource = None
        self._service_annotation_disallowed_list = None
        self._service_label_disallowed_list = None
        self.discriminator = None

        self.explainers = explainers
        if resource is not None:
            self.resource = resource
        if service_annotation_disallowed_list is not None:
            self.service_annotation_disallowed_list = service_annotation_disallowed_list
        if service_label_disallowed_list is not None:
            self.service_label_disallowed_list = service_label_disallowed_list

    @property
    def explainers(self):
        """Gets the explainers of this V1beta1InferenceServicesConfig.  # noqa: E501


        :return: The explainers of this V1beta1InferenceServicesConfig.  # noqa: E501
        :rtype: V1beta1ExplainersConfig
        """
        return self._explainers

    @explainers.setter
    def explainers(self, explainers):
        """Sets the explainers of this V1beta1InferenceServicesConfig.


        :param explainers: The explainers of this V1beta1InferenceServicesConfig.  # noqa: E501
        :type: V1beta1ExplainersConfig
        """
        if self.local_vars_configuration.client_side_validation and explainers is None:  # noqa: E501
            raise ValueError("Invalid value for `explainers`, must not be `None`")  # noqa: E501

        self._explainers = explainers

    @property
    def resource(self):
        """Gets the resource of this V1beta1InferenceServicesConfig.  # noqa: E501


        :return: The resource of this V1beta1InferenceServicesConfig.  # noqa: E501
        :rtype: V1beta1ResourceConfig
        """
        return self._resource

    @resource.setter
    def resource(self, resource):
        """Sets the resource of this V1beta1InferenceServicesConfig.


        :param resource: The resource of this V1beta1InferenceServicesConfig.  # noqa: E501
        :type: V1beta1ResourceConfig
        """

        self._resource = resource

    @property
    def service_annotation_disallowed_list(self):
        """Gets the service_annotation_disallowed_list of this V1beta1InferenceServicesConfig.  # noqa: E501

        ServiceAnnotationDisallowedList is a list of annotations that are not allowed to be propagated to Knative revisions  # noqa: E501

        :return: The service_annotation_disallowed_list of this V1beta1InferenceServicesConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._service_annotation_disallowed_list

    @service_annotation_disallowed_list.setter
    def service_annotation_disallowed_list(self, service_annotation_disallowed_list):
        """Sets the service_annotation_disallowed_list of this V1beta1InferenceServicesConfig.

        ServiceAnnotationDisallowedList is a list of annotations that are not allowed to be propagated to Knative revisions  # noqa: E501

        :param service_annotation_disallowed_list: The service_annotation_disallowed_list of this V1beta1InferenceServicesConfig.  # noqa: E501
        :type: list[str]
        """

        self._service_annotation_disallowed_list = service_annotation_disallowed_list

    @property
    def service_label_disallowed_list(self):
        """Gets the service_label_disallowed_list of this V1beta1InferenceServicesConfig.  # noqa: E501

        ServiceLabelDisallowedList is a list of labels that are not allowed to be propagated to Knative revisions  # noqa: E501

        :return: The service_label_disallowed_list of this V1beta1InferenceServicesConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._service_label_disallowed_list

    @service_label_disallowed_list.setter
    def service_label_disallowed_list(self, service_label_disallowed_list):
        """Sets the service_label_disallowed_list of this V1beta1InferenceServicesConfig.

        ServiceLabelDisallowedList is a list of labels that are not allowed to be propagated to Knative revisions  # noqa: E501

        :param service_label_disallowed_list: The service_label_disallowed_list of this V1beta1InferenceServicesConfig.  # noqa: E501
        :type: list[str]
        """

        self._service_label_disallowed_list = service_label_disallowed_list

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1InferenceServicesConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1InferenceServicesConfig):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: grpc_predict_v2.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15grpc_predict_v2.proto\x12\tinference\"\x13\n\x11ServerLiveRequest\"\"\n\x12ServerLiveResponse\x12\x0c\n\x04live\x18\x01 \x01(\x08\"\x14\n\x12ServerReadyRequest\"$\n\x13ServerReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\"2\n\x11ModelReadyRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"#\n\x12ModelReadyResponse\x12\r\n\x05ready\x18\x01 \x01(\x08\"\x17\n\x15ServerMetadataRequest\"K\n\x16ServerMetadataResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\x12\x12\n\nextensions\x18\x03 \x03(\t\"5\n\x14ModelMetadataRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\t\"\x8d\x02\n\x15ModelMetadataResponse\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08versions\x18\x02 \x03(\t\x12\x10\n\x08platform\x18\x03 \x01(\t\x12?\n\x06inputs\x18\x04 \x03(\x0b\x32/.inference.ModelMetadataResponse.TensorMetadata\x12@\n\x07outputs\x18\x05 \x03(\x0b\x32/.inference.ModelMetadataResponse.TensorMetadata\x1a?\n\x0eTensorMetadata\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tatype\x18\x02 \x01(\t\x12\r\n\x05shape\x18\x03 \x03(\x03\"\xee\x06\n\x11ModelInferRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x15\n\rmodel_version\x18\x02 \x01(\t\x12\n\n\x02id\x18\x03 \x01(\t\x12@\n\nparameters\x18\x04 \x03(\x0b\x32,.inference.ModelInferRequest.ParametersEntry\x12=\n\x06inputs\x18\x05 \x03(\x0b\x32-.inference.ModelInferRequest.InferInputTensor\x12H\n\x07outputs\x18\x06 \x03(\x0b\x32\x37.inference.ModelInferRequest.InferRequestedOutputTensor\x12\x1a\n\x12raw_input_contents\x18\x07 \x03(\x0c\x1a\x94\x02\n\x10InferInputTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tatype\x18\x02 \x01(\t\x12\r\n\x05shape\x18\x03 \x03(\x03\x12Q\n\nparameters\x18\x04 \x03(\x0b\x32=.inference.ModelInferRequest.InferInputTensor.ParametersEntry\x12\x30\n\x08\x63ontents\x18\x05 \x01(\x0b\x32\x1e.inference.InferTensorContents\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\x1a\xd5\x01\n\x1aInferRequestedOutputTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12[\n\nparameters\x18\x02 \x03(\x0b\x32G.inference.ModelInferRequest.InferRequestedOutputTensor.ParametersEntry\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\"\xd5\x04\n\x12ModelInferResponse\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x15\n\rmodel_version\x18\x02 \x01(\t\x12\n\n\x02id\x18\x03 \x01(\t\x12\x41\n\nparameters\x18\x04 \x03(\x0b\x32-.inference.ModelInferResponse.ParametersEntry\x12@\n\x07outputs\x18\x05 \x03(\x0b\x32/.inference.ModelInferResponse.InferOutputTensor\x12\x1b\n\x13raw_output_contents\x18\x06 \x03(\x0c\x1a\x97\x02\n\x11InferOutputTensor\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08\x64\x61tatype\x18\x02 \x01(\t\x12\r\n\x05shape\x18\x03 \x03(\x03\x12S\n\nparameters\x18\x04 \x03(\x0b\x32?.inference.ModelInferResponse.InferOutputTensor.ParametersEntry\x12\x30\n\x08\x63ontents\x18\x05 \x01(\x0b\x32\x1e.inference.InferTensorContents\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\x1aL\n\x0fParametersEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12(\n\x05value\x18\x02 \x01(\x0b\x32\x19.inference.InferParameter:\x02\x38\x01\"i\n\x0eInferParameter\x12\x14\n\nbool_param\x18\x01 \x01(\x08H\x00\x12\x15\n\x0bint64_param\x18\x02 \x01(\x03H\x00\x12\x16\n\x0cstring_param\x18\x03 \x01(\tH\x00\x42\x12\n\x10parameter_choice\"\xd0\x01\n\x13InferTensorContents\x12\x15\n\rbool_contents\x18\x01 \x03(\x08\x12\x14\n\x0cint_contents\x18\x02 \x03(\x05\x12\x16\n\x0eint64_contents\x18\x03 \x03(\x03\x12\x15\n\ruint_contents\x18\x04 \x03(\r\x12\x17\n\x0fuint64_contents\x18\x05 \x03(\x04\x12\x15\n\rfp32_contents\x18\x06 \x03(\x02\x12\x15\n\rfp64_contents\x18\x07 \x03(\x01\x12\x16\n\x0e\x62ytes_contents\x18\x08 \x03(\x0c\"0\n\x1aRepositoryModelLoadRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\"C\n\x1bRepositoryModelLoadResponse\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x10\n\x08isLoaded\x18\x02 \x01(\x08\"2\n\x1cRepositoryModelUnloadRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\"G\n\x1dRepositoryModelUnloadResponse\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x12\n\nisUnloaded\x18\x02 \x01(\x08\x32\xd2\x05\n\x14GRPCInferenceService\x12K\n\nServerLive\x12\x1c.inference.ServerLiveRequest\x1a\x1d.inference.ServerLiveResponse\"\x00\x12N\n\x0bServerReady\x12\x1d.inference.ServerReadyRequest\x1a\x1e.inference.ServerReadyResponse\"\x00\x12K\n\nModelReady\x12\x1c.inference.ModelReadyRequest\x1a\x1d.inference.ModelReadyResponse\"\x00\x12W\n\x0eServerMetadata\x12 .inference.ServerMetadataRequest\x1a!.inference.ServerMetadataResponse\"\x00\x12T\n\rModelMetadata\x12\x1f.inference.ModelMetadataRequest\x1a .inference.ModelMetadataResponse\"\x00\x12K\n\nModelInfer\x12\x1c.inference.ModelInferRequest\x1a\x1d.inference.ModelInferResponse\"\x00\x12\x66\n\x13RepositoryModelLoad\x12%.inference.RepositoryModelLoadRequest\x1a&.inference.RepositoryModelLoadResponse\"\x00\x12l\n\x15RepositoryModelUnload\x12\'.inference.RepositoryModelUnloadRequest\x1a(.inference.RepositoryModelUnloadResponse\"\x00\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'grpc_predict_v2_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY._options = None
  _MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY._serialized_options = b'8\001'
  _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY._options = None
  _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY._serialized_options = b'8\001'
  _MODELINFERREQUEST_PARAMETERSENTRY._options = None
  _MODELINFERREQUEST_PARAMETERSENTRY._serialized_options = b'8\001'
  _MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY._options = None
  _MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY._serialized_options = b'8\001'
  _MODELINFERRESPONSE_PARAMETERSENTRY._options = None
  _MODELINFERRESPONSE_PARAMETERSENTRY._serialized_options = b'8\001'
  _SERVERLIVEREQUEST._serialized_start=36
  _SERVERLIVEREQUEST._serialized_end=55
  _SERVERLIVERESPONSE._serialized_start=57
  _SERVERLIVERESPONSE._serialized_end=91
  _SERVERREADYREQUEST._serialized_start=93
  _SERVERREADYREQUEST._serialized_end=113
  _SERVERREADYRESPONSE._serialized_start=115
  _SERVERREADYRESPONSE._serialized_end=151
  _MODELREADYREQUEST._serialized_start=153
  _MODELREADYREQUEST._serialized_end=203
  _MODELREADYRESPONSE._serialized_start=205
  _MODELREADYRESPONSE._serialized_end=240
  _SERVERMETADATAREQUEST._serialized_start=242
  _SERVERMETADATAREQUEST._serialized_end=265
  _SERVERMETADATARESPONSE._serialized_start=267
  _SERVERMETADATARESPONSE._serialized_end=342
  _MODELMETADATAREQUEST._serialized_start=344
  _MODELMETADATAREQUEST._serialized_end=397
  _MODELMETADATARESPONSE._serialized_start=400
  _MODELMETADATARESPONSE._serialized_end=669
  _MODELMETADATARESPONSE_TENSORMETADATA._serialized_start=606
  _MODELMETADATARESPONSE_TENSORMETADATA._serialized_end=669
  _MODELINFERREQUEST._serialized_start=672
  _MODELINFERREQUEST._serialized_end=1550
  _MODELINFERREQUEST_INFERINPUTTENSOR._serialized_start=980
  _MODELINFERREQUEST_INFERINPUTTENSOR._serialized_end=1256
  _MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY._serialized_start=1180
  _MODELINFERREQUEST_INFERINPUTTENSOR_PARAMETERSENTRY._serialized_end=1256
  _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR._serialized_start=1259
  _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR._serialized_end=1472
  _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY._serialized_start=1180
  _MODELINFERREQUEST_INFERREQUESTEDOUTPUTTENSOR_PARAMETERSENTRY._serialized_end=1256
  _MODELINFERREQUEST_PARAMETERSENTRY._serialized_start=1180
  _MODELINFERREQUEST_PARAMETERSENTRY._serialized_end=1256
  _MODELINFERRESPONSE._serialized_start=1553
  _MODELINFERRESPONSE._serialized_end=2150
  _MODELINFERRESPONSE_INFEROUTPUTTENSOR._serialized_start=1793
  _MODELINFERRESPONSE_INFEROUTPUTTENSOR._serialized_end=2072
  _MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY._serialized_start=1180
  _MODELINFERRESPONSE_INFEROUTPUTTENSOR_PARAMETERSENTRY._serialized_end=1256
  _MODELINFERRESPONSE_PARAMETERSENTRY._serialized_start=1180
  _MODELINFERRESPONSE_PARAMETERSENTRY._serialized_end=1256
  _INFERPARAMETER._serialized_start=2152
  _INFERPARAMETER._serialized_end=2257
  _INFERTENSORCONTENTS._serialized_start=2260
  _INFERTENSORCONTENTS._serialized_end=2468
  _REPOSITORYMODELLOADREQUEST._serialized_start=2470
  _REPOSITORYMODELLOADREQUEST._serialized_end=2518
  _REPOSITORYMODELLOADRESPONSE._serialized_start=2520
  _REPOSITORYMODELLOADRESPONSE._serialized_end=2587
  _REPOSITORYMODELUNLOADREQUEST._serialized_start=2589
  _REPOSITORYMODELUNLOADREQUEST._serialized_end=2639
  _REPOSITORYMODELUNLOADRESPONSE._serialized_start=2641
  _REPOSITORYMODELUNLOADRESPONSE._serialized_end=2712
  _GRPCINFERENCESERVICE._serialized_start=2715
  _GRPCINFERENCESERVICE._serialized_end=3437
# @@protoc_insertion_point(module_scope)



# Copyright 2021 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
from enum import Enum

# KServe K8S constants
KSERVE_GROUP = "serving.kserve.io"
KSERVE_KIND_INFERENCESERVICE = "InferenceService"
KSERVE_PLURAL_INFERENCESERVICE = "inferenceservices"
KSERVE_KIND_TRAINEDMODEL = "TrainedModel"
KSERVE_PLURAL_TRAINEDMODEL = "trainedmodels"
KSERVE_KIND_INFERENCEGRAPH = "InferenceGraph"
KSERVE_PLURAL_INFERENCEGRAPH = "inferencegraphs"
KSERVE_KIND_LOCALMODELNODEGROUP = "LocalModelNodeGroup"
KSERVE_PLURAL_LOCALMODELNODEGROUP = "localmodelnodegroups"
KSERVE_KIND_LOCALMODELCACHE = "LocalModelCache"
KSERVE_PLURAL_LOCALMODELCACHE = "localmodelcaches"
KSERVE_KIND_LOCALMODELNODE = "LocalModelNode"
KSERVE_PLURAL_LOCALMODELNODE = "localmodelnodes"
KSERVE_V1BETA1_VERSION = "v1beta1"
KSERVE_V1ALPHA1_VERSION = "v1alpha1"

KSERVE_V1BETA1 = KSERVE_GROUP + "/" + KSERVE_V1BETA1_VERSION
KSERVE_V1ALPHA1 = KSERVE_GROUP + "/" + KSERVE_V1ALPHA1_VERSION

KSERVE_LOGLEVEL = os.environ.get("KSERVE_LOGLEVEL", "INFO").upper()

# INFERENCESERVICE credentials common constants
INFERENCESERVICE_CONFIG_MAP_NAME = "inferenceservice-config"
INFERENCESERVICE_SYSTEM_NAMESPACE = "kserve"
DEFAULT_SECRET_NAME = "kserve-secret-"
DEFAULT_SA_NAME = "kserve-service-credentials"

# S3 credentials constants
S3_ACCESS_KEY_ID_DEFAULT_NAME = "AWS_ACCESS_KEY_ID"
S3_SECRET_ACCESS_KEY_DEFAULT_NAME = "AWS_SECRET_ACCESS_KEY"
S3_DEFAULT_CREDS_FILE = "~/.aws/credentials"

# GCS credentials constants
GCS_CREDS_FILE_DEFAULT_NAME = "gcloud-application-credentials.json"
GCS_DEFAULT_CREDS_FILE = "~/.config/gcloud/application_default_credentials.json"

# Azure credentials constants
AZ_DEFAULT_CREDS_FILE = "~/.azure/azure_credentials.json"

# Model Serve Constants
KSERVE_MODEL_SERVER_NAME = "kserve"

# GRPC content datatype mappings constants
GRPC_CONTENT_DATATYPE_MAPPINGS = {
    "BOOL": "bool_contents",
    "INT8": "int_contents",
    "INT16": "int_contents",
    "INT32": "int_contents",
    "INT64": "int64_contents",
    "UINT8": "uint_contents",
    "UINT16": "uint_contents",
    "UINT32": "uint_contents",
    "UINT64": "uint64_contents",
    "FP32": "fp32_contents",
    "FP64": "fp64_contents",
    "BYTES": "bytes_contents",
}
# K8S status key constants
OBSERVED_GENERATION = "observedGeneration"

# K8S metadata key constants
GENERATION = "generation"

EXPLAINER_BASE_URL_FORMAT = "{0}://{1}"


class PredictorProtocol(Enum):
    REST_V1 = "v1"
    REST_V2 = "v2"
    GRPC_V2 = "grpc-v2"


# LLM stats map key
LLM_STATS_KEY = "llm-stats"

# Default GRPC max message length
MAX_GRPC_MESSAGE_LENGTH = 8388608

V2_ROUTE_PREFIX = "/v2"
V1_ROUTE_PREFIX = "/v1"

DEFAULT_HTTP_PORT = 8080
DEFAULT_GRPC_PORT = 8081

# Header containing the json length in case of REST raw response.
INFERENCE_CONTENT_LENGTH_HEADER = "inference-header-content-length"


class ModelType(Enum):
    EXPLAINER = 1
    PREDICTOR = 2



# Copyright 2022 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1beta1PredictorsConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'lightgbm': 'V1beta1PredictorConfig',
        'onnx': 'V1beta1PredictorConfig',
        'paddle': 'V1beta1PredictorConfig',
        'pmml': 'V1beta1PredictorConfig',
        'pytorch': 'V1beta1PredictorConfig',
        'sklearn': 'V1beta1PredictorProtocols',
        'tensorflow': 'V1beta1PredictorConfig',
        'triton': 'V1beta1PredictorConfig',
        'xgboost': 'V1beta1PredictorProtocols'
    }

    attribute_map = {
        'lightgbm': 'lightgbm',
        'onnx': 'onnx',
        'paddle': 'paddle',
        'pmml': 'pmml',
        'pytorch': 'pytorch',
        'sklearn': 'sklearn',
        'tensorflow': 'tensorflow',
        'triton': 'triton',
        'xgboost': 'xgboost'
    }

    def __init__(self, lightgbm=None, onnx=None, paddle=None, pmml=None, pytorch=None, sklearn=None, tensorflow=None, triton=None, xgboost=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1PredictorsConfig - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._lightgbm = None
        self._onnx = None
        self._paddle = None
        self._pmml = None
        self._pytorch = None
        self._sklearn = None
        self._tensorflow = None
        self._triton = None
        self._xgboost = None
        self.discriminator = None

        if lightgbm is not None:
            self.lightgbm = lightgbm
        if onnx is not None:
            self.onnx = onnx
        if paddle is not None:
            self.paddle = paddle
        if pmml is not None:
            self.pmml = pmml
        if pytorch is not None:
            self.pytorch = pytorch
        if sklearn is not None:
            self.sklearn = sklearn
        if tensorflow is not None:
            self.tensorflow = tensorflow
        if triton is not None:
            self.triton = triton
        if xgboost is not None:
            self.xgboost = xgboost

    @property
    def lightgbm(self):
        """Gets the lightgbm of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The lightgbm of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorConfig
        """
        return self._lightgbm

    @lightgbm.setter
    def lightgbm(self, lightgbm):
        """Sets the lightgbm of this V1beta1PredictorsConfig.


        :param lightgbm: The lightgbm of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorConfig
        """

        self._lightgbm = lightgbm

    @property
    def onnx(self):
        """Gets the onnx of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The onnx of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorConfig
        """
        return self._onnx

    @onnx.setter
    def onnx(self, onnx):
        """Sets the onnx of this V1beta1PredictorsConfig.


        :param onnx: The onnx of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorConfig
        """

        self._onnx = onnx

    @property
    def paddle(self):
        """Gets the paddle of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The paddle of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorConfig
        """
        return self._paddle

    @paddle.setter
    def paddle(self, paddle):
        """Sets the paddle of this V1beta1PredictorsConfig.


        :param paddle: The paddle of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorConfig
        """

        self._paddle = paddle

    @property
    def pmml(self):
        """Gets the pmml of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The pmml of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorConfig
        """
        return self._pmml

    @pmml.setter
    def pmml(self, pmml):
        """Sets the pmml of this V1beta1PredictorsConfig.


        :param pmml: The pmml of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorConfig
        """

        self._pmml = pmml

    @property
    def pytorch(self):
        """Gets the pytorch of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The pytorch of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorConfig
        """
        return self._pytorch

    @pytorch.setter
    def pytorch(self, pytorch):
        """Sets the pytorch of this V1beta1PredictorsConfig.


        :param pytorch: The pytorch of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorConfig
        """

        self._pytorch = pytorch

    @property
    def sklearn(self):
        """Gets the sklearn of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The sklearn of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorProtocols
        """
        return self._sklearn

    @sklearn.setter
    def sklearn(self, sklearn):
        """Sets the sklearn of this V1beta1PredictorsConfig.


        :param sklearn: The sklearn of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorProtocols
        """

        self._sklearn = sklearn

    @property
    def tensorflow(self):
        """Gets the tensorflow of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The tensorflow of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorConfig
        """
        return self._tensorflow

    @tensorflow.setter
    def tensorflow(self, tensorflow):
        """Sets the tensorflow of this V1beta1PredictorsConfig.


        :param tensorflow: The tensorflow of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorConfig
        """

        self._tensorflow = tensorflow

    @property
    def triton(self):
        """Gets the triton of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The triton of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorConfig
        """
        return self._triton

    @triton.setter
    def triton(self, triton):
        """Sets the triton of this V1beta1PredictorsConfig.


        :param triton: The triton of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorConfig
        """

        self._triton = triton

    @property
    def xgboost(self):
        """Gets the xgboost of this V1beta1PredictorsConfig.  # noqa: E501


        :return: The xgboost of this V1beta1PredictorsConfig.  # noqa: E501
        :rtype: V1beta1PredictorProtocols
        """
        return self._xgboost

    @xgboost.setter
    def xgboost(self, xgboost):
        """Sets the xgboost of this V1beta1PredictorsConfig.


        :param xgboost: The xgboost of this V1beta1PredictorsConfig.  # noqa: E501
        :type: V1beta1PredictorProtocols
        """

        self._xgboost = xgboost

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1PredictorsConfig):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1PredictorsConfig):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import struct
from typing import Optional, List, Dict, Union, Tuple

import numpy as np
import orjson
import pandas as pd
import uuid
from google.protobuf.internal.containers import MessageMap

from .rest.v2_datamodels import InferenceRequest
from ..constants.constants import GRPC_CONTENT_DATATYPE_MAPPINGS
from ..errors import InvalidInput, InferenceError
from ..protocol.grpc.grpc_predict_v2_pb2 import (
    ModelInferRequest,
    InferTensorContents,
    ModelInferResponse,
    InferParameter,
)
from ..utils.numpy_codec import to_np_dtype, from_np_dtype


def serialize_byte_tensor(input_tensor: np.ndarray) -> np.ndarray:
    """
    Serializes a bytes tensor into a flat numpy array of length prepended
    bytes. The numpy array should use dtype of np.object. For np.bytes,
    numpy will remove trailing zeros at the end of byte sequence and because
    of this it should be avoided.

    Args:
        input_tensor : np.array
            The bytes tensor to serialize.
    Returns:
        serialized_bytes_tensor : np.array
            The 1-D numpy array of type uint8 containing the serialized bytes in row-major form.
    Raises:
        InferenceError If unable to serialize the given tensor.
    """

    if input_tensor.size == 0:
        return np.empty([0], dtype=np.object_)

    # If the input is a tensor of string/bytes objects, then must flatten those into
    # a 1-dimensional array containing the 4-byte byte size followed by the
    # actual element bytes. All elements are concatenated together in row-major
    # order.

    if (input_tensor.dtype != np.object_) and (input_tensor.dtype.type != np.bytes_):
        raise InferenceError("cannot serialize bytes tensor: invalid datatype")

    flattened_ls = []
    # 'C' order is row-major.
    for obj in np.nditer(input_tensor, flags=["refs_ok"], order="C"):
        # If directly passing bytes to BYTES type,
        # don't convert it to str as Python will encode the
        # bytes which may distort the meaning
        if input_tensor.dtype == np.object_:
            if type(obj.item()) == bytes:
                s = obj.item()
            else:
                s = str(obj.item()).encode("utf-8")
        else:
            s = obj.item()
        flattened_ls.append(struct.pack("<I", len(s)))
        flattened_ls.append(s)
    flattened = b"".join(flattened_ls)
    flattened_array = np.asarray(flattened, dtype=np.object_)
    if not flattened_array.flags["C_CONTIGUOUS"]:
        flattened_array = np.ascontiguousarray(flattened_array, dtype=np.object_)
    return flattened_array


def deserialize_bytes_tensor(encoded_tensor: bytes) -> np.ndarray:
    """
    Deserializes an encoded bytes tensor into a
    numpy array of dtype of python objects

    Args:
        encoded_tensor : bytes
            The encoded bytes tensor where each element
            has its length in first 4 bytes followed by
            the content
    Returns:
        string_tensor : np.array
            The 1-D numpy array of type object containing the
            deserialized bytes in row-major form.
    """
    strs = list()
    offset = 0
    val_buf = encoded_tensor
    while offset < len(val_buf):
        length = struct.unpack_from("<I", val_buf, offset)[0]
        offset += 4
        sb = struct.unpack_from("<{}s".format(length), val_buf, offset)[0]
        offset += length
        strs.append(sb)
    return np.array(strs, dtype=np.object_)


class InferInput:
    _name: str
    _shape: List[int]
    _datatype: str
    _parameters: Dict

    def __init__(
        self,
        name: str,
        shape: List[int],
        datatype: str,
        data: Union[List, np.ndarray, InferTensorContents] = None,
        parameters: Optional[Union[Dict, MessageMap[str, InferParameter]]] = None,
    ):
        """An object of InferInput class is used to describe the input tensor of an inference request.

        Args:
            name: The name of the inference input whose data will be described by this object.
            shape : The shape of the associated inference input.
            datatype : The data type of the associated inference input.
            data : The data of the inference input.
                   When data is not set, raw_data is used for gRPC to transmit with numpy array bytes
                   by using `set_data_from_numpy`.
            parameters : The additional inference parameters.
        """

        self._name = name
        self._shape = shape
        self._datatype = datatype.upper()
        self._parameters = parameters
        self._data = data
        self._raw_data = None

    @property
    def name(self) -> str:
        """Get the name of inference input associated with this object.

        Returns:
            The name of the inference input
        """
        return self._name

    @property
    def datatype(self) -> str:
        """Get the datatype of inference input associated with this object.

        Returns:
            The datatype of the inference input.
        """
        return self._datatype

    @property
    def data(self) -> Union[List, np.ndarray, InferTensorContents]:
        """Get the data of the inference input associated with this object.

        Returns:
            The data of the inference input.
        """
        return self._data

    @data.setter
    def data(self, data: List):
        """Set the data of the inference input associated with this object.

        Args:
             data: data of the inference input.
        """
        self._data = data

    @property
    def shape(self) -> List[int]:
        """Get the shape of inference input associated with this object.

        Returns:
            The shape of the inference input.
        """
        return self._shape

    @property
    def parameters(self) -> Union[Dict, MessageMap[str, InferParameter], None]:
        """Get the parameters of the inference input associated with this object.

        Returns:
            The additional inference parameters
        """
        return self._parameters

    @parameters.setter
    def parameters(
        self, params: Optional[Union[Dict, MessageMap[str, InferParameter]]]
    ):
        """Set the parameters of the inference input associated with this object.

        Args:
             params: parameters of the inference input
        """
        self._parameters = params

    @shape.setter
    def shape(self, shape: List[int]):
        """Set the shape of inference input.

        Args:
            shape : The shape of the associated inference input.
        """
        self._shape = shape

    def as_string(self) -> List[List[str]]:
        """
        Decodes the inference input data as a list of strings.

        Returns:
            List[List[str]]: The decoded data as a list of strings.

        Raises:
            InvalidInput: If the datatype of the inference input is not 'BYTES'.
        """
        if self.datatype == "BYTES":
            return [s.decode("utf-8") for li in self._data for s in li]
        else:
            raise InvalidInput(f"invalid datatype {self.datatype} in the input")

    def as_numpy(self) -> np.ndarray:
        """Decode the inference input data as numpy array.

        Returns:
            A numpy array of the inference input data
        Raises:
            InvalidInput: If the datatype of the inference input is not recognized.
        """
        dtype = to_np_dtype(self.datatype)
        if dtype is None:
            raise InvalidInput(f"invalid datatype {dtype} in the input")
        if self._raw_data is not None:
            if self.datatype == "BYTES":
                # String results contain a 4-byte string length
                # followed by the actual string characters. Hence,
                # need to decode the raw bytes to convert into
                # array elements.
                np_array = deserialize_bytes_tensor(self._raw_data)
            else:
                np_array = np.frombuffer(self._raw_data, dtype=dtype)
            return np_array.reshape(self._shape)
        else:
            np_array = np.array(self._data, dtype=dtype)
            return np_array.reshape(self._shape)

    def set_data_from_numpy(self, input_tensor: np.ndarray, binary_data: bool = True):
        """Set the tensor data from the specified numpy array for input associated with this object.

        Args:
            input_tensor : The tensor data in numpy array format.
            binary_data : Indicates whether to set data for the input in binary format
                          or explicit tensor within JSON. The default value is True,
                          which means the data will be delivered as binary data with gRPC or in the
                          HTTP body after the JSON object for REST.

        Raises:
            InferenceError if failed to set data for the tensor.
        """
        if not isinstance(input_tensor, (np.ndarray,)):
            raise InferenceError("input_tensor must be a numpy array")

        dtype = from_np_dtype(input_tensor.dtype)
        if self._datatype != dtype:
            raise InferenceError(
                "got unexpected datatype {} from numpy array, expected {}".format(
                    dtype, self._datatype
                )
            )
        valid_shape = True
        if len(self._shape) != len(input_tensor.shape):
            valid_shape = False
        else:
            for i in range(len(self._shape)):
                if self._shape[i] != input_tensor.shape[i]:
                    valid_shape = False
        if not valid_shape:
            raise InferenceError(
                "got unexpected numpy array shape [{}], expected [{}]".format(
                    str(input_tensor.shape)[1:-1], str(self._shape)[1:-1]
                )
            )

        if not binary_data:
            if self._parameters:
                self._parameters.pop("binary_data_size", None)
            self._raw_data = None
            if self._datatype == "BYTES":
                self._data = []
                try:
                    if input_tensor.size > 0:
                        for obj in np.nditer(
                            input_tensor, flags=["refs_ok"], order="C"
                        ):
                            # We need to convert the object to string using utf-8,
                            # if we want to use the binary_data=False. JSON requires
                            # the input to be a UTF-8 string.
                            if input_tensor.dtype == np.object_:
                                if type(obj.item()) == bytes:
                                    self._data.append(str(obj.item(), encoding="utf-8"))
                                else:
                                    self._data.append(str(obj.item()))
                            else:
                                self._data.append(str(obj.item(), encoding="utf-8"))
                except UnicodeDecodeError:
                    raise InferenceError(
                        f'Failed to encode "{obj.item()}" using UTF-8. Please use binary_data=True, if'
                        " you want to pass a byte array."
                    )
            else:
                self._data = [val.item() for val in input_tensor.flatten()]
        else:
            self._data = None
            if self._datatype == "BYTES":
                serialized_output = serialize_byte_tensor(input_tensor)
                if serialized_output.size > 0:
                    self._raw_data = serialized_output.item()
                else:
                    self._raw_data = b""
            else:
                self._raw_data = input_tensor.tobytes()
            if self._parameters is None:
                self._parameters = {"binary_data_size": len(self._raw_data)}
            else:
                self._parameters["binary_data_size"] = len(self._raw_data)

    def __eq__(self, other):
        if not isinstance(other, InferInput):
            return False
        if self.name != other.name:
            return False
        if self.shape != other.shape:
            return False
        if self.datatype != other.datatype:
            return False
        if self.parameters != other.parameters:
            return False
        if self.data != other.data:
            return False
        if self._raw_data != other._raw_data:
            return False
        return True

    @parameters.setter
    def parameters(self, value):
        self._parameters = value

    def to_dict(self) -> dict:
        return {
            "name": self.name,
            "shape": self.shape,
            "datatype": self.datatype,
            "data": self.data,
            "parameters": self.parameters,
        }

    def __repr__(self) -> str:
        return (
            f'"name": "{self.name}",'
            f'"shape": {self.shape},'
            f'"datatype": "{self.datatype}",'
            f'"data": {self.data},'
            f'"parameters": {self.parameters}'
        )

    def __str__(self) -> str:
        return self.__repr__()


def get_content(datatype: str, data: InferTensorContents):
    if datatype == "BOOL":
        return list(data.bool_contents)
    elif datatype in ["UINT8", "UINT16", "UINT32"]:
        return list(data.uint_contents)
    elif datatype == "UINT64":
        return list(data.uint64_contents)
    elif datatype in ["INT8", "INT16", "INT32"]:
        return list(data.int_contents)
    elif datatype == "INT64":
        return list(data.int64_contents)
    elif datatype == "FP16":
        # FP16 data should be present in raw_input_content, so return an empty list.
        return list()
    elif datatype == "FP32":
        return list(data.fp32_contents)
    elif datatype == "FP64":
        return list(data.fp64_contents)
    elif datatype == "BYTES":
        return list(data.bytes_contents)
    else:
        raise InvalidInput("invalid content type")


class RequestedOutput:
    def __init__(self, name: str, parameters: Optional[Dict] = None):
        """
        The RequestedOutput class represents an output that is requested as part of an inference request.

        Args:
            name (str): The name of the output.
            parameters (Optional[Dict]): Additional parameters for the output.
        """
        self._name = name
        self._parameters = parameters

    @property
    def name(self) -> str:
        """
        Get the name of the output.

        Returns:
            str: The name of the output.
        """
        return self._name

    @property
    def parameters(self) -> Optional[Dict]:
        """
        Get the additional parameters for the output.

        Returns:
            Optional[Dict]: The additional parameters for the output.
        """
        return self._parameters

    @parameters.setter
    def parameters(
        self, params: Optional[Union[Dict, MessageMap[str, InferParameter]]]
    ):
        """Set the parameters of the inference input associated with this object.

        Args:
             params: parameters of the inference input
        """
        self._parameters = params

    @property
    def binary_data(self) -> Optional[bool]:
        """Get the binary_data attribute from the parameters.
        This attribute indicates whether the data for the input should be in binary format.

        Returns:
            bool or None: True if the data should be in binary format, False otherwise.
                              If the binary_data attribute is not set, returns None.
        """
        if self.parameters and "binary_data" in self.parameters:
            return self.parameters["binary_data"]
        else:
            return None

    def __eq__(self, other):
        if not isinstance(other, RequestedOutput):
            return False
        if self.name != other.name:
            return False
        if self.parameters != other.parameters:
            return False
        return True

    def __repr__(self):
        return f"RequestedOutput(name={self.name}, parameters={self.parameters})"

    def __str__(self):
        return self.__repr__()


class InferRequest:
    id: Optional[str]
    model_name: str
    parameters: Optional[Dict]
    inputs: List[InferInput]
    from_grpc: bool

    def __init__(
        self,
        model_name: str,
        infer_inputs: List[InferInput],
        request_id: Optional[str] = None,
        raw_inputs=None,
        from_grpc: Optional[bool] = False,
        parameters: Optional[Union[Dict, MessageMap[str, InferParameter]]] = None,
        request_outputs: Optional[List[RequestedOutput]] = None,
    ):
        """InferRequest Data Model.

        Args:
            model_name: The model name.
            infer_inputs: The inference inputs for the model.
            request_id: The id for the inference request.
            raw_inputs: The binary data for the inference inputs.
            from_grpc: Indicate if the data model is constructed from gRPC request.
            parameters: The additional inference parameters.
            request_outputs: The output tensors requested for this inference.
        """

        self.id = request_id
        self.model_name = model_name
        self.inputs = infer_inputs
        self.parameters = parameters
        self.from_grpc = from_grpc
        self._use_raw_outputs = False
        if raw_inputs:
            self._use_raw_outputs = True
            for i, raw_input in enumerate(raw_inputs):
                self.inputs[i]._raw_data = raw_input
        self.request_outputs = request_outputs

    @property
    def use_binary_outputs(self) -> bool:
        """This attribute is used to determine if all the outputs should be returned as raw binary format.
        For REST,
            Get the binary_data_output attribute from the parameters. This will be ovverided by the individual output's 'binary_data' parameter.
        For GRPC,
            It is True, if the received inputs are raw_inputs, otherwise False. For GRPC, if the inputs are raw_inputs,
            then the outputs should be returned as raw_outputs.

        Returns:
            a boolean indicating whether to use binary raw outputs
        """
        # If the request is from gRPC and we receive the inputs as raw inputs, then the outputs should be returned as raw binary format.
        if self._use_raw_outputs and self.from_grpc:
            return True
        # If the request is from REST and the 'use_binary_outputs' parameter is set to True, then the outputs should be returned as raw binary format.
        elif self.parameters and not self.from_grpc:
            # If it is a grpc request, then this configuration has no effect on the output.
            return self.parameters.get("binary_data_output", False)
        else:
            return False

    @classmethod
    def from_grpc(cls, request: ModelInferRequest) -> "InferRequest":
        """
        Class method to construct an InferRequest object from a ModelInferRequest object.

        Args:
            request (ModelInferRequest): The gRPC ModelInferRequest object to be converted.

        Returns:
            InferRequest: The resulting InferRequest object.
        """
        infer_inputs = [
            InferInput(
                name=input_tensor.name,
                shape=list(input_tensor.shape),
                datatype=input_tensor.datatype,
                data=get_content(input_tensor.datatype, input_tensor.contents),
                parameters=(
                    to_http_parameters(input_tensor.parameters)
                    if input_tensor.parameters
                    else None
                ),
            )
            for input_tensor in request.inputs
        ]
        request_outputs = [
            RequestedOutput(
                name=output.name,
                parameters=(
                    to_http_parameters(output.parameters) if output.parameters else None
                ),
            )
            for output in request.outputs
        ]
        return cls(
            request_id=request.id,
            model_name=request.model_name,
            infer_inputs=infer_inputs,
            raw_inputs=request.raw_input_contents,
            from_grpc=True,
            parameters=request.parameters,
            request_outputs=request_outputs if request_outputs else None,
        )

    @classmethod
    def from_bytes(
        cls, req_bytes: bytes, json_length: int, model_name: str
    ) -> "InferRequest":
        """The class method to construct the InferRequest object from REST raw request bytes.

        Args:
            req_bytes (bytes): The raw InferRequest in bytes.
            json_length (int): The length of the json bytes.
            model_name (str): The name of the model.

        Returns:
            InferRequest: The resulting InferRequest object.

        Raises:
            InvalidInput: If the request format is unrecognized or if necessary fields are missing.
        """
        json_bytes = req_bytes[:json_length]
        try:
            infer_req_dict = orjson.loads(json_bytes)
        except orjson.JSONDecodeError as e:
            raise InvalidInput(f"Unrecognized request format: {e}")
        infer_inputs = []
        # Read the raw binary inputs appended after json
        start_index = json_length
        for input_ in infer_req_dict["inputs"]:
            parameters = input_.get("parameters", None)
            infer_input = InferInput(
                name=input_["name"],
                shape=input_["shape"],
                datatype=input_["datatype"],
                parameters=parameters,
            )
            infer_input_data = input_.get("data", None)
            if infer_input_data is not None:
                if infer_input.datatype == "FP16":
                    raise InvalidInput(
                        f"Receiving FP16 data via JSON is not supported. Please use the binary data format "
                        f"for input {infer_input.name}"
                    )
                infer_input.data = infer_input_data
            elif parameters and "binary_data_size" in parameters:
                binary_data_size = parameters.get("binary_data_size", None)
                if binary_data_size is None:
                    raise InvalidInput(
                        f"'binary_data_size' is not specified for input '{infer_input.name}' for model '{model_name}'"
                    )
                end_index = start_index + binary_data_size
                infer_input._raw_data = req_bytes[start_index:end_index]
                infer_input.set_data_from_numpy(
                    infer_input.as_numpy(), binary_data=False
                )
                start_index = end_index
            else:
                raise InvalidInput(
                    f"'data' field is missing for input '{infer_input.name}' for model '{model_name}'"
                )
            infer_inputs.append(infer_input)
        requested_outputs = None
        if infer_req_dict.get("outputs", None) is not None:
            requested_outputs = [
                RequestedOutput(
                    name=output["name"],
                    parameters=output.get("parameters", None),
                )
                for output in infer_req_dict["outputs"]
            ]
        return cls(
            model_name=model_name,
            request_id=infer_req_dict.get("id", None),
            parameters=infer_req_dict.get("parameters", None),
            infer_inputs=infer_inputs,
            request_outputs=requested_outputs,
        )

    @classmethod
    def from_inference_request(
        cls, request: InferenceRequest, model_name: str
    ) -> "InferRequest":
        """The class method to construct the InferRequest object from InferenceRequest object.

        Args:
            request (InferenceRequest): The InferenceRequest object.
            model_name (str): The name of the model.
        Returns:
            InferRequest: The resulting InferRequest object.
        Raises:
            InvalidInput: If the request format is unrecognized.
        """
        infer_inputs = []
        for infer_input in request.inputs:
            if infer_input.datatype == "FP16" and len(infer_input.data) != 0:
                raise InvalidInput(
                    f"Sending FP16 data via JSON is not supported. "
                    f"Please use the binary data format for input {infer_input.name}"
                )
            infer_inputs.append(
                InferInput(
                    name=infer_input.name,
                    shape=infer_input.shape,
                    datatype=infer_input.datatype,
                    data=infer_input.data,
                    parameters=(
                        {} if infer_input.parameters is None else infer_input.parameters
                    ),
                )
            )

        requested_outputs = None
        if request.outputs:
            requested_outputs = [
                RequestedOutput(
                    name=output.name,
                    parameters=({} if output.parameters is None else output.parameters),
                )
                for output in request.outputs
            ]
        return cls(
            request_id=request.id,
            model_name=model_name,
            infer_inputs=infer_inputs,
            parameters=request.parameters,
            request_outputs=requested_outputs,
        )

    def to_rest(self) -> Tuple[Union[bytes, Dict], Optional[int]]:
        """
        Converts the InferRequest object to v2 REST InferRequest Dict or bytes.
        This method is used to convert the InferRequest object into a format that can be sent over a REST API.

        Returns:
            Tuple[Union[bytes, Dict], Optional[int]]: A tuple containing the InferRequest in bytes or Dict and the length of the JSON part of the request.
                                                      If it is dict, then the json length will be None.

        Raises:
            InvalidInput: If the data is missing for an input or if both 'data' and 'raw_data' fields are set for an input.
        """
        infer_inputs = []
        raw_inputs = []
        for infer_input in self.inputs:
            if infer_input.data is None and infer_input._raw_data is None:
                raise InvalidInput(
                    f"'data' field is missing for output '{infer_input.name}' for model '{self.model_name}'"
                )
            if isinstance(infer_input.data, np.ndarray):
                infer_input.set_data_from_numpy(infer_input.data, binary_data=False)
            if infer_input.data and infer_input._raw_data:
                raise InvalidInput(
                    f"Both 'data' and 'raw_data' fields are set for input '{infer_input.name}' for model '{self.model_name}'"
                )

            if infer_input.datatype == "FP16" and infer_input.data:
                raise InvalidInput(
                    f"Sending FP16 data via JSON is not supported. Please use the binary data format for input {infer_input.name}"
                )

            infer_input_dict = {
                "name": infer_input.name,
                "shape": infer_input.shape,
                "datatype": infer_input.datatype,
            }
            if infer_input.parameters:
                infer_input_dict["parameters"] = to_http_parameters(
                    infer_input.parameters
                )
            if infer_input._raw_data:
                raw_inputs.append(infer_input._raw_data)
            else:
                infer_input_dict["data"] = infer_input.data
            infer_inputs.append(infer_input_dict)
        requested_outputs = []
        if self.request_outputs:
            for requested_output in self.request_outputs:
                requested_output_dict = {
                    "name": requested_output.name,
                }
                if requested_output.parameters:
                    requested_output_dict["parameters"] = to_http_parameters(
                        requested_output.parameters
                    )
                requested_outputs.append(requested_output_dict)
        res = {
            "id": self.id if self.id else str(uuid.uuid4()),
            "model_name": self.model_name,
            "inputs": infer_inputs,
        }
        if requested_outputs:
            res["outputs"] = requested_outputs
        if self.parameters:
            res["parameters"] = to_http_parameters(self.parameters)

        if len(raw_inputs) != 0:
            infer_response_bytes = orjson.dumps(res)
            json_length = len(infer_response_bytes)
            infer_response_bytes = b"".join([infer_response_bytes] + raw_inputs)
            return infer_response_bytes, json_length

        return res, None

    def to_grpc(self) -> ModelInferRequest:
        """Converts the InferRequest object to gRPC ModelInferRequest type.

        Returns:
            ModelInferRequest gRPC type converted from InferRequest object.
        """
        infer_inputs = []
        raw_input_contents = []
        for infer_input in self.inputs:
            if isinstance(infer_input.data, np.ndarray):
                infer_input.set_data_from_numpy(infer_input.data, binary_data=True)
            infer_input_dict = {
                "name": infer_input.name,
                "shape": infer_input.shape,
                "datatype": infer_input.datatype,
            }
            if infer_input.parameters:
                infer_input_dict["parameters"] = to_grpc_parameters(
                    infer_input.parameters
                )
            if infer_input._raw_data is not None:
                raw_input_contents.append(infer_input._raw_data)
            else:
                if not isinstance(infer_input.data, List):
                    raise InvalidInput("input data is not a List")
                infer_input_dict["contents"] = {}
                data_key = GRPC_CONTENT_DATATYPE_MAPPINGS.get(
                    infer_input.datatype, None
                )
                if data_key is not None:
                    infer_input._data = [
                        bytes(val, "utf-8") if isinstance(val, str) else val
                        for val in infer_input.data
                    ]  # str to byte conversion for grpc proto
                    infer_input_dict["contents"][data_key] = infer_input.data
                else:
                    raise InvalidInput("invalid input datatype")
            infer_inputs.append(infer_input_dict)
        request_outputs = []
        if self.request_outputs:
            for request_output in self.request_outputs:
                request_output_dict = {"name": request_output.name}
                if request_output.parameters:
                    request_output_dict["parameters"] = to_grpc_parameters(
                        request_output.parameters
                    )
                request_outputs.append(request_output_dict)

        return ModelInferRequest(
            id=self.id,
            model_name=self.model_name,
            inputs=infer_inputs,
            raw_input_contents=raw_input_contents,
            parameters=to_grpc_parameters(self.parameters) if self.parameters else None,
            outputs=request_outputs if request_outputs else None,
        )

    def as_dataframe(self) -> pd.DataFrame:
        """Decode the tensor inputs as pandas dataframe.

        Returns:
            The inference input data as pandas dataframe
        """
        dfs = []
        for input in self.inputs:
            input_data = input.data
            if input.datatype == "BYTES":
                input_data = [
                    str(val, "utf-8") if isinstance(val, bytes) else val
                    for val in input.data
                ]
            dfs.append(pd.DataFrame(input_data, columns=[input.name]))
        return pd.concat(dfs, axis=1)

    def get_input_by_name(self, name: str) -> Optional[InferInput]:
        """Find an input Tensor in the InferenceRequest that has the given name
        Args:
            name : str
                name of the input Tensor object
        Returns:
            InferInput
                The InferInput with the specified name, or None if no
                input with this name exists
        """
        for infer_input in self.inputs:
            if name == infer_input.name:
                return infer_input
        return None

    def __eq__(self, other):
        if not isinstance(other, InferRequest):
            return False
        if self.model_name != other.model_name:
            return False
        if self.id != other.id:
            return False
        if self.from_grpc != other.from_grpc:
            return False
        if self.parameters != other.parameters:
            return False
        if self.inputs != other.inputs:
            return False
        if self.request_outputs != other.request_outputs:
            return False
        return True

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "model_name": self.model_name,
            "inputs": [infer_input.to_dict() for infer_input in self.inputs],
            "parameters": self.parameters,
            "from_grpc": self.from_grpc,
        }

    def __repr__(self) -> str:
        return (
            f'"id": "{self.id}",'
            f'"model_name": "{self.model_name}",'
            f'"inputs": {self.inputs.__repr__()},'
            f'"parameters": {self.parameters},'
            f'"from_grpc": {self.from_grpc}'
        )

    def __str__(self) -> str:
        return self.__repr__()


class InferOutput:
    def __init__(
        self,
        name: str,
        shape: List[int],
        datatype: str,
        data: Union[List, np.ndarray, InferTensorContents] = None,
        parameters: Optional[Union[Dict, MessageMap[str, InferParameter]]] = None,
    ):
        """An object of InferOutput class is used to describe the output tensor for an inference response.

        Args:
            name : The name of inference output whose data will be described by this object.
            shape : The shape of the associated inference output.
            datatype : The data type of the associated inference output.
            data : The data of the inference output. When data is not set,
                   raw_data is used for gRPC with numpy array bytes by calling set_data_from_numpy.
            parameters : The additional inference parameters.
        """

        self._name = name
        self._shape = shape
        self._datatype = datatype.upper()
        self._parameters = parameters
        self._data = data
        self._raw_data = None

    @property
    def name(self) -> str:
        """Get the name of inference output associated with this object.

        Returns:
            The name of inference output.
        """
        return self._name

    @property
    def datatype(self) -> str:
        """Get the data type of inference output associated with this object.

        Returns:
            The data type of inference output.
        """
        return self._datatype

    @property
    def data(self) -> Union[List, np.ndarray, InferTensorContents]:
        """Get the data of inference output associated with this object.

        Returns:
            The data of inference output.
        """
        return self._data

    @data.setter
    def data(self, data: Union[List, np.ndarray, InferTensorContents]):
        """Set the data of inference output associated with this object.

        Args:
            data: inference output data.
        """
        self._data = data

    @property
    def shape(self) -> List[int]:
        """Get the shape of inference output associated with this object.

        Returns:
            The shape of inference output
        """
        return self._shape

    @shape.setter
    def shape(self, shape: List[int]):
        """Set the shape of inference output.

        Args:
            shape: The shape of the associated inference output.
        """
        self._shape = shape

    @property
    def parameters(self) -> Union[Dict, MessageMap[str, InferParameter], None]:
        """Get the parameters of inference output associated with this object.

        Returns:
            The additional inference parameters associated with the inference output.
        """
        return self._parameters

    @parameters.setter
    def parameters(self, params: Union[Dict, MessageMap[str, InferParameter]]):
        """Set the parameters of inference output associated with this object.

        :param params: The parameters of inference output
        """
        self._parameters = params

    def as_numpy(self) -> np.ndarray:
        """Decode the tensor output data as numpy array.

        Returns:
            The numpy array of the associated inference output data.
        """
        dtype = to_np_dtype(self.datatype)
        if dtype is None:
            raise InvalidInput("invalid datatype in the input")
        if self._raw_data is not None:
            if self.datatype == "BYTES":
                # String results contain a 4-byte string length
                # followed by the actual string characters. Hence,
                # need to decode the raw bytes to convert into
                # array elements.
                np_array = deserialize_bytes_tensor(self._raw_data)
            else:
                np_array = np.frombuffer(self._raw_data, dtype=dtype)
            return np_array.reshape(self._shape)
        else:
            np_array = np.array(self._data, dtype=dtype)
            return np_array.reshape(self._shape)

    def set_data_from_numpy(self, output_tensor: np.ndarray, binary_data: bool = True):
        """Set the tensor data from the specified numpy array for the inference output associated with this object.

        Args:
            output_tensor : The tensor data in numpy array format.
            binary_data : Indicates whether to set data for the input in binary format
                          or explicit tensor within JSON. The default value is True,
                          which means the data will be delivered as binary data with gRPC or in the
                          HTTP body after the JSON object for REST.

        Raises:
            InferenceError if failed to set data for the output tensor.
        """
        if not isinstance(output_tensor, (np.ndarray,)):
            raise InferenceError("input_tensor must be a numpy array")

        dtype = from_np_dtype(output_tensor.dtype)
        if np.issubdtype(output_tensor.dtype, np.datetime64):
            output_tensor = output_tensor.astype(np.object_)
        if self._datatype != dtype:
            raise InferenceError(
                "got unexpected datatype {} from numpy array, expected {}".format(
                    dtype, self._datatype
                )
            )
        valid_shape = True
        if len(self._shape) != len(output_tensor.shape):
            valid_shape = False
        else:
            for i in range(len(self._shape)):
                if self._shape[i] != output_tensor.shape[i]:
                    valid_shape = False
        if not valid_shape:
            raise InferenceError(
                "got unexpected numpy array shape [{}], expected [{}]".format(
                    str(output_tensor.shape)[1:-1], str(self._shape)[1:-1]
                )
            )

        if not binary_data:
            if self._parameters:
                self._parameters.pop("binary_data_size", None)
            self._raw_data = None
            if self._datatype == "BYTES":
                self._data = []
                try:
                    if output_tensor.size > 0:
                        for obj in np.nditer(
                            output_tensor, flags=["refs_ok"], order="C"
                        ):
                            # We need to convert the object to string using utf-8,
                            # if we want to use the binary_data=False. JSON requires
                            # the input to be a UTF-8 string.
                            if output_tensor.dtype == np.object_:
                                if type(obj.item()) == bytes:
                                    self._data.append(str(obj.item(), encoding="utf-8"))
                                else:
                                    self._data.append(str(obj.item()))
                            else:
                                self._data.append(str(obj.item(), encoding="utf-8"))
                except UnicodeDecodeError:
                    raise InferenceError(
                        f'Failed to encode "{obj.item()}" using UTF-8. Please use binary_data=True, if'
                        " you want to pass a byte array."
                    )
            else:
                self._data = [val.item() for val in output_tensor.flatten()]
        else:
            self._data = None
            if self._datatype == "BYTES":
                serialized_output = serialize_byte_tensor(output_tensor)
                if serialized_output.size > 0:
                    self._raw_data = serialized_output.item()
                else:
                    self._raw_data = b""
            else:
                self._raw_data = output_tensor.tobytes()
            if self._parameters is None:
                self._parameters = {"binary_data_size": len(self._raw_data)}
            else:
                self._parameters["binary_data_size"] = len(self._raw_data)

    def __eq__(self, other):
        if not isinstance(other, InferOutput):
            return False
        if self.name != other.name:
            return False
        if self.shape != other.shape:
            return False
        if self.datatype != other.datatype:
            return False
        if self.parameters != other.parameters:
            return False
        if self.data != other.data:
            return False
        if self._raw_data != other._raw_data:
            return False
        return True

    def to_dict(self) -> dict:
        return {
            "name": self.name,
            "shape": self.shape,
            "datatype": self.datatype,
            "data": self.data,
            "parameters": self.parameters,
        }

    def __repr__(self) -> str:
        return (
            f'"name": "{self.name}",'
            f'"shape": {self.shape},'
            f'"datatype": "{self.datatype}",'
            f'"data": {self.data},'
            f'"parameters": {self.parameters}'
        )

    def __str__(self) -> str:
        return self.__repr__()


class InferResponse:
    id: str
    model_name: str
    model_version: Optional[str]
    parameters: Optional[Dict]
    outputs: List[InferOutput]
    from_grpc: bool

    def __init__(
        self,
        response_id: str,
        model_name: str,
        infer_outputs: List[InferOutput],
        model_version: Optional[str] = None,
        raw_outputs=None,
        from_grpc: Optional[bool] = False,
        parameters: Optional[Union[Dict, MessageMap[str, InferParameter]]] = None,
        use_binary_outputs: Optional[bool] = False,
        requested_outputs: Optional[List[RequestedOutput]] = None,
    ):
        """The InferResponse Data Model

        Args:
            response_id: The id of the inference response.
            model_name: The name of the model.
            infer_outputs: The inference outputs of the inference response.
            model_version: The version of the model.
            raw_outputs: The raw binary data of the inference outputs.
            from_grpc: Indicate if the InferResponse is constructed from a gRPC response.
            parameters: The additional inference parameters.
            use_binary_outputs: A boolean indicating whether the data for the outputs should be in binary format when sent over REST API.
                                This will be overridden by the individual output's binary_data attribute.
            requested_outputs: The output tensors requested for this inference.
        """

        self.id = response_id
        self.model_name = model_name
        self.model_version = model_version
        self.outputs = infer_outputs
        self.parameters = parameters
        self.from_grpc = from_grpc
        self._requested_outputs = requested_outputs
        self._use_binary_outputs: bool = use_binary_outputs
        if raw_outputs:
            for i, raw_output in enumerate(raw_outputs):
                self.outputs[i]._raw_data = raw_output

    @classmethod
    def from_grpc(cls, response: ModelInferResponse) -> "InferResponse":
        """The class method to construct the InferResponse object from gRPC message type.

        Args:
            response: The GRPC response as ModelInferResponse object.
        Returns:
            InferResponse object.
        """
        infer_outputs = [
            InferOutput(
                name=output.name,
                shape=list(output.shape),
                datatype=output.datatype,
                data=get_content(output.datatype, output.contents),
                parameters=output.parameters,
            )
            for output in response.outputs
        ]
        return cls(
            model_name=response.model_name,
            model_version=response.model_version,
            response_id=response.id,
            parameters=response.parameters,
            infer_outputs=infer_outputs,
            raw_outputs=response.raw_output_contents,
            from_grpc=True,
        )

    @classmethod
    def from_rest(cls, response: Dict) -> "InferResponse":
        """The class method to construct the InferResponse object from REST message type.

        Args:
            response: The response as a dict.
        Returns:
            InferResponse object.
        """
        infer_outputs = [
            InferOutput(
                name=output["name"],
                shape=list(output["shape"]),
                datatype=output["datatype"],
                data=output["data"],
                parameters=output.get("parameters", None),
            )
            for output in response["outputs"]
        ]
        return cls(
            model_name=response.get("model_name"),
            model_version=response.get("model_version", None),
            response_id=response.get("id", None),
            parameters=response.get("parameters", None),
            infer_outputs=infer_outputs,
        )

    @classmethod
    def from_bytes(
        cls,
        res_bytes: bytes,
        json_length: int,
    ) -> "InferResponse":
        """
        Class method to construct an InferResponse object from raw response bytes.
        This method is used to convert the raw response bytes received from a REST API into an InferResponse object.

        Args:
            res_bytes (bytes): The raw response bytes received from the REST API.
            json_length (int): The length of the JSON part of the response.

        Returns:
            InferResponse: The constructed InferResponse object.

        Raises:
            InvalidInput: If the response format is unrecognized or if necessary fields are missing in the response.
            InferenceError: if failed to set data for the output tensor.
        """
        # If json_length is equal to the length of the response bytes, then the response does not have
        # any appended binary data after the json.
        json_bytes = res_bytes[:json_length]
        try:
            infer_res_dict = orjson.loads(json_bytes)
        except orjson.JSONDecodeError as e:
            raise InvalidInput(f"Unrecognized request format: {e}")
        model_name = infer_res_dict["model_name"]
        infer_outputs = []
        # Read the raw binary outputs appended after json
        start_index = json_length
        for output in infer_res_dict["outputs"]:
            parameters = output.get("parameters", None)
            infer_output = InferOutput(
                name=output["name"],
                shape=output["shape"],
                datatype=output["datatype"],
                parameters=parameters,
            )
            if parameters and "binary_data_size" in parameters:
                binary_data_size = parameters.get("binary_data_size")
                end_index = start_index + binary_data_size
                infer_output._raw_data = res_bytes[start_index:end_index]
                infer_output.set_data_from_numpy(
                    infer_output.as_numpy(), binary_data=False
                )
                start_index = end_index
            else:
                infer_output_data = output.get("data", None)
                if infer_output_data is None:
                    raise InvalidInput(
                        f"'data' field is missing for output '{infer_output.name}' for model '{model_name}'"
                    )
                infer_output.data = infer_output_data
            infer_outputs.append(infer_output)
        return cls(
            model_name=model_name,
            response_id=infer_res_dict.get("id", None),
            parameters=infer_res_dict.get("parameters", None),
            infer_outputs=infer_outputs,
        )

    def to_rest(self) -> Tuple[Union[bytes, Dict], Optional[int]]:
        """
        Converts the InferResponse object to v2 REST InferResponse Dict or bytes.
        This method is used to convert the InferResponse object into a format that can be sent over a REST API.

        Returns:
            Tuple[Union[bytes, Dict], Optional[int]]: A tuple containing the InferResponse in bytes or Dict and the length of the JSON part of the response.
                                                      If it is dict, then the json length will be None.

        Raises:
            InvalidInput: If the output data is not a numpy array, bytes, or list.
        """
        infer_outputs = []
        raw_outputs = []
        use_binary_data = self._use_binary_outputs
        outputs = self._requested_outputs if self._requested_outputs else self.outputs

        for output in outputs:
            infer_output = (
                self.get_output_by_name(output.name)
                if self._requested_outputs
                else output
            )
            if self._requested_outputs:
                use_binary_data = output.binary_data
            if infer_output is None:
                raise InvalidInput(
                    f"Unexpected inference output '{output.name}' for model '{self.model_name}'"
                )
            if infer_output.data is None and infer_output._raw_data is None:
                raise InvalidInput(
                    f"'data' field is missing for output '{infer_output.name}' for model '{self.model_name}'"
                )
            if isinstance(infer_output.data, np.ndarray):
                infer_output.set_data_from_numpy(
                    infer_output.data, binary_data=use_binary_data
                )
            elif infer_output.data or infer_output._raw_data:
                infer_output.set_data_from_numpy(
                    infer_output.as_numpy(), binary_data=use_binary_data
                )
            if infer_output.datatype == "FP16" and infer_output.data:
                raise InvalidInput(
                    f"Sending FP16 data via JSON is not supported. Please use the binary data format for output {infer_output.name}"
                )

            infer_output_dict = {
                "name": infer_output.name,
                "shape": infer_output.shape,
                "datatype": infer_output.datatype,
            }
            if infer_output.parameters:
                infer_output_dict["parameters"] = to_http_parameters(
                    infer_output.parameters
                )
            if use_binary_data:
                raw_outputs.append(infer_output._raw_data)
            else:
                infer_output_dict["data"] = infer_output.data
            infer_outputs.append(infer_output_dict)

        res = {
            "id": self.id,
            "model_name": self.model_name,
            "model_version": self.model_version,
            "outputs": infer_outputs,
        }
        if self.parameters:
            res["parameters"] = to_http_parameters(self.parameters)

        if len(raw_outputs) != 0:
            infer_response_bytes = orjson.dumps(res)
            json_length = len(infer_response_bytes)
            infer_response_bytes = b"".join([infer_response_bytes] + raw_outputs)
            return infer_response_bytes, json_length

        return res, None

    def to_grpc(self) -> ModelInferResponse:
        """Converts the InferResponse object to gRPC ModelInferResponse type.

        Returns:
            The ModelInferResponse gRPC message.
        Raises:
            InvalidInput: If the output data is not a List or if the datatype is invalid.
        """
        infer_outputs = []
        raw_output_contents = []
        use_raw_outputs = self._use_binary_outputs
        if not self._use_binary_outputs:
            # If FP16 datatype is present in the outputs use raw outputs.
            if _contains_fp16_datatype(self):
                use_raw_outputs = True
        for infer_output in self.outputs:
            if (
                use_raw_outputs
                and infer_output.data
                and isinstance(infer_output.data, list)
            ):
                infer_output.data = infer_output.as_numpy()
            if isinstance(infer_output.data, np.ndarray):
                infer_output.set_data_from_numpy(infer_output.data, binary_data=True)
            infer_output_dict = {
                "name": infer_output.name,
                "shape": infer_output.shape,
                "datatype": infer_output.datatype,
            }
            if infer_output.parameters:
                infer_output_dict["parameters"] = to_grpc_parameters(
                    infer_output.parameters
                )
            if infer_output._raw_data is not None:
                raw_output_contents.append(infer_output._raw_data)
            else:
                if not isinstance(infer_output.data, List):
                    raise InvalidInput("output data is not a List")
                infer_output_dict["contents"] = {}
                data_key = GRPC_CONTENT_DATATYPE_MAPPINGS.get(
                    infer_output.datatype, None
                )
                if data_key is not None:
                    infer_output._data = [
                        bytes(val, "utf-8") if isinstance(val, str) else val
                        for val in infer_output.data
                    ]  # str to byte conversion for grpc proto
                    infer_output_dict["contents"][data_key] = infer_output.data
                else:
                    raise InvalidInput("to_grpc: invalid output datatype")
            infer_outputs.append(infer_output_dict)

        return ModelInferResponse(
            id=self.id,
            model_name=self.model_name,
            model_version=self.model_version,
            outputs=infer_outputs,
            raw_output_contents=raw_output_contents,
            parameters=to_grpc_parameters(self.parameters) if self.parameters else None,
        )

    def get_output_by_name(self, name: str) -> Optional[InferOutput]:
        """Find an output Tensor in the InferResponse that has the given name

        Args:
            name : str
                name of the output Tensor object
        Returns:
            InferOutput
                The InferOutput with the specified name, or None if no
                output with this name exists
        """
        for infer_output in self.outputs:
            if name == infer_output.name:
                return infer_output
        return None

    def __eq__(self, other):
        if not isinstance(other, InferResponse):
            return False
        if self.model_name != other.model_name:
            return False
        if self.model_version != other.model_version:
            return False
        if self.id != other.id:
            return False
        if self.from_grpc != other.from_grpc:
            return False
        if self.parameters != other.parameters:
            return False
        if self.outputs != other.outputs:
            return False
        return True

    def to_dict(self) -> dict:
        return {
            "id": self.id,
            "model_name": self.model_name,
            "outputs": [infer_output.to_dict() for infer_output in self.outputs],
            "parameters": self.parameters,
            "from_grpc": self.from_grpc,
        }

    def __repr__(self) -> str:
        return (
            f'"id": "{self.id}",'
            f'"model_name": "{self.model_name}",'
            f'"outputs": {self.outputs.__repr__()},'
            f'"parameters": {self.parameters},'
            f'"from_grpc": {self.from_grpc}'
        )

    def __str__(self) -> str:
        return self.__repr__()


def to_grpc_parameters(
    parameters: Union[Dict[str, Union[str, bool, int]], MessageMap[str, InferParameter]]
) -> Dict[str, InferParameter]:
    """
    Converts REST parameters to GRPC InferParameter objects

    :param parameters: parameters to be converted.
    :return: converted parameters as Dict[str, InferParameter]
    :raises InvalidInput: if the parameter type is not supported.
    """
    grpc_params: Dict[str, InferParameter] = {}
    for key, val in parameters.items():
        if isinstance(val, str):
            grpc_params[key] = InferParameter(string_param=val)
        elif isinstance(val, bool):
            grpc_params[key] = InferParameter(bool_param=val)
        elif isinstance(val, int):
            grpc_params[key] = InferParameter(int64_param=val)
        elif isinstance(val, InferParameter):
            grpc_params[key] = val
        else:
            raise InvalidInput(f"to_grpc: invalid parameter value: {val}")
    return grpc_params


def to_http_parameters(
    parameters: Union[dict, MessageMap[str, InferParameter]]
) -> Dict[str, Union[str, bool, int]]:
    """
    Converts GRPC InferParameter parameters to REST parameters

    :param parameters: parameters to be converted.
    :return: converted parameters as Dict[str, Union[str, bool, int]]
    """
    http_params: Dict[str, Union[str, bool, int]] = {}
    for key, val in parameters.items():
        if isinstance(val, InferParameter):
            if val.HasField("bool_param"):
                http_params[key] = val.bool_param
            elif val.HasField("int64_param"):
                http_params[key] = val.int64_param
            elif val.HasField("string_param"):
                http_params[key] = val.string_param
        else:
            http_params[key] = val
    return http_params


def _contains_fp16_datatype(infer_response: InferResponse) -> bool:
    """
    Checks whether the InferResponse outputs contains FP16 datatype.

    :param infer_response: An InferResponse object containing model inference results.
    :return: A boolean indicating whether any output in the InferResponse uses the FP16 datatype.
    """
    for infer_output in infer_response.outputs:
        if infer_output.datatype == "FP16":
            return True
    return False



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1alpha1ServingRuntimeSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'affinity': 'V1Affinity',
        'annotations': 'dict(str, str)',
        'built_in_adapter': 'V1alpha1BuiltInAdapter',
        'containers': 'list[V1Container]',
        'disabled': 'bool',
        'grpc_data_endpoint': 'str',
        'grpc_endpoint': 'str',
        'host_ipc': 'bool',
        'http_data_endpoint': 'str',
        'image_pull_secrets': 'list[V1LocalObjectReference]',
        'labels': 'dict(str, str)',
        'multi_model': 'bool',
        'node_selector': 'dict(str, str)',
        'protocol_versions': 'list[str]',
        'replicas': 'int',
        'storage_helper': 'V1alpha1StorageHelper',
        'supported_model_formats': 'list[V1alpha1SupportedModelFormat]',
        'tolerations': 'list[V1Toleration]',
        'volumes': 'list[V1Volume]',
        'worker_spec': 'V1alpha1WorkerSpec'
    }

    attribute_map = {
        'affinity': 'affinity',
        'annotations': 'annotations',
        'built_in_adapter': 'builtInAdapter',
        'containers': 'containers',
        'disabled': 'disabled',
        'grpc_data_endpoint': 'grpcDataEndpoint',
        'grpc_endpoint': 'grpcEndpoint',
        'host_ipc': 'hostIPC',
        'http_data_endpoint': 'httpDataEndpoint',
        'image_pull_secrets': 'imagePullSecrets',
        'labels': 'labels',
        'multi_model': 'multiModel',
        'node_selector': 'nodeSelector',
        'protocol_versions': 'protocolVersions',
        'replicas': 'replicas',
        'storage_helper': 'storageHelper',
        'supported_model_formats': 'supportedModelFormats',
        'tolerations': 'tolerations',
        'volumes': 'volumes',
        'worker_spec': 'workerSpec'
    }

    def __init__(self, affinity=None, annotations=None, built_in_adapter=None, containers=None, disabled=None, grpc_data_endpoint=None, grpc_endpoint=None, host_ipc=None, http_data_endpoint=None, image_pull_secrets=None, labels=None, multi_model=None, node_selector=None, protocol_versions=None, replicas=None, storage_helper=None, supported_model_formats=None, tolerations=None, volumes=None, worker_spec=None, local_vars_configuration=None):  # noqa: E501
        """V1alpha1ServingRuntimeSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._affinity = None
        self._annotations = None
        self._built_in_adapter = None
        self._containers = None
        self._disabled = None
        self._grpc_data_endpoint = None
        self._grpc_endpoint = None
        self._host_ipc = None
        self._http_data_endpoint = None
        self._image_pull_secrets = None
        self._labels = None
        self._multi_model = None
        self._node_selector = None
        self._protocol_versions = None
        self._replicas = None
        self._storage_helper = None
        self._supported_model_formats = None
        self._tolerations = None
        self._volumes = None
        self._worker_spec = None
        self.discriminator = None

        if affinity is not None:
            self.affinity = affinity
        if annotations is not None:
            self.annotations = annotations
        if built_in_adapter is not None:
            self.built_in_adapter = built_in_adapter
        self.containers = containers
        if disabled is not None:
            self.disabled = disabled
        if grpc_data_endpoint is not None:
            self.grpc_data_endpoint = grpc_data_endpoint
        if grpc_endpoint is not None:
            self.grpc_endpoint = grpc_endpoint
        if host_ipc is not None:
            self.host_ipc = host_ipc
        if http_data_endpoint is not None:
            self.http_data_endpoint = http_data_endpoint
        if image_pull_secrets is not None:
            self.image_pull_secrets = image_pull_secrets
        if labels is not None:
            self.labels = labels
        if multi_model is not None:
            self.multi_model = multi_model
        if node_selector is not None:
            self.node_selector = node_selector
        if protocol_versions is not None:
            self.protocol_versions = protocol_versions
        if replicas is not None:
            self.replicas = replicas
        if storage_helper is not None:
            self.storage_helper = storage_helper
        if supported_model_formats is not None:
            self.supported_model_formats = supported_model_formats
        if tolerations is not None:
            self.tolerations = tolerations
        if volumes is not None:
            self.volumes = volumes
        if worker_spec is not None:
            self.worker_spec = worker_spec

    @property
    def affinity(self):
        """Gets the affinity of this V1alpha1ServingRuntimeSpec.  # noqa: E501


        :return: The affinity of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: V1Affinity
        """
        return self._affinity

    @affinity.setter
    def affinity(self, affinity):
        """Sets the affinity of this V1alpha1ServingRuntimeSpec.


        :param affinity: The affinity of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: V1Affinity
        """

        self._affinity = affinity

    @property
    def annotations(self):
        """Gets the annotations of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Annotations that will be add to the pod. More info: http://kubernetes.io/docs/user-guide/annotations  # noqa: E501

        :return: The annotations of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._annotations

    @annotations.setter
    def annotations(self, annotations):
        """Sets the annotations of this V1alpha1ServingRuntimeSpec.

        Annotations that will be add to the pod. More info: http://kubernetes.io/docs/user-guide/annotations  # noqa: E501

        :param annotations: The annotations of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._annotations = annotations

    @property
    def built_in_adapter(self):
        """Gets the built_in_adapter of this V1alpha1ServingRuntimeSpec.  # noqa: E501


        :return: The built_in_adapter of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: V1alpha1BuiltInAdapter
        """
        return self._built_in_adapter

    @built_in_adapter.setter
    def built_in_adapter(self, built_in_adapter):
        """Sets the built_in_adapter of this V1alpha1ServingRuntimeSpec.


        :param built_in_adapter: The built_in_adapter of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: V1alpha1BuiltInAdapter
        """

        self._built_in_adapter = built_in_adapter

    @property
    def containers(self):
        """Gets the containers of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        List of containers belonging to the pod. Containers cannot currently be added or removed. There must be at least one container in a Pod. Cannot be updated.  # noqa: E501

        :return: The containers of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: list[V1Container]
        """
        return self._containers

    @containers.setter
    def containers(self, containers):
        """Sets the containers of this V1alpha1ServingRuntimeSpec.

        List of containers belonging to the pod. Containers cannot currently be added or removed. There must be at least one container in a Pod. Cannot be updated.  # noqa: E501

        :param containers: The containers of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: list[V1Container]
        """
        if self.local_vars_configuration.client_side_validation and containers is None:  # noqa: E501
            raise ValueError("Invalid value for `containers`, must not be `None`")  # noqa: E501

        self._containers = containers

    @property
    def disabled(self):
        """Gets the disabled of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Set to true to disable use of this runtime  # noqa: E501

        :return: The disabled of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: bool
        """
        return self._disabled

    @disabled.setter
    def disabled(self, disabled):
        """Sets the disabled of this V1alpha1ServingRuntimeSpec.

        Set to true to disable use of this runtime  # noqa: E501

        :param disabled: The disabled of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: bool
        """

        self._disabled = disabled

    @property
    def grpc_data_endpoint(self):
        """Gets the grpc_data_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Grpc endpoint for inferencing  # noqa: E501

        :return: The grpc_data_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: str
        """
        return self._grpc_data_endpoint

    @grpc_data_endpoint.setter
    def grpc_data_endpoint(self, grpc_data_endpoint):
        """Sets the grpc_data_endpoint of this V1alpha1ServingRuntimeSpec.

        Grpc endpoint for inferencing  # noqa: E501

        :param grpc_data_endpoint: The grpc_data_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: str
        """

        self._grpc_data_endpoint = grpc_data_endpoint

    @property
    def grpc_endpoint(self):
        """Gets the grpc_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Grpc endpoint for internal model-management (implementing mmesh.ModelRuntime gRPC service) Assumed to be single-model runtime if omitted  # noqa: E501

        :return: The grpc_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: str
        """
        return self._grpc_endpoint

    @grpc_endpoint.setter
    def grpc_endpoint(self, grpc_endpoint):
        """Sets the grpc_endpoint of this V1alpha1ServingRuntimeSpec.

        Grpc endpoint for internal model-management (implementing mmesh.ModelRuntime gRPC service) Assumed to be single-model runtime if omitted  # noqa: E501

        :param grpc_endpoint: The grpc_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: str
        """

        self._grpc_endpoint = grpc_endpoint

    @property
    def host_ipc(self):
        """Gets the host_ipc of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Use the host's ipc namespace. Optional: Default to false.  # noqa: E501

        :return: The host_ipc of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: bool
        """
        return self._host_ipc

    @host_ipc.setter
    def host_ipc(self, host_ipc):
        """Sets the host_ipc of this V1alpha1ServingRuntimeSpec.

        Use the host's ipc namespace. Optional: Default to false.  # noqa: E501

        :param host_ipc: The host_ipc of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: bool
        """

        self._host_ipc = host_ipc

    @property
    def http_data_endpoint(self):
        """Gets the http_data_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        HTTP endpoint for inferencing  # noqa: E501

        :return: The http_data_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: str
        """
        return self._http_data_endpoint

    @http_data_endpoint.setter
    def http_data_endpoint(self, http_data_endpoint):
        """Sets the http_data_endpoint of this V1alpha1ServingRuntimeSpec.

        HTTP endpoint for inferencing  # noqa: E501

        :param http_data_endpoint: The http_data_endpoint of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: str
        """

        self._http_data_endpoint = http_data_endpoint

    @property
    def image_pull_secrets(self):
        """Gets the image_pull_secrets of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        ImagePullSecrets is an optional list of references to secrets in the same namespace to use for pulling any of the images used by this PodSpec. If specified, these secrets will be passed to individual puller implementations for them to use. For example, in the case of docker, only DockerConfig type secrets are honored. More info: https://kubernetes.io/docs/concepts/containers/images#specifying-imagepullsecrets-on-a-pod  # noqa: E501

        :return: The image_pull_secrets of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: list[V1LocalObjectReference]
        """
        return self._image_pull_secrets

    @image_pull_secrets.setter
    def image_pull_secrets(self, image_pull_secrets):
        """Sets the image_pull_secrets of this V1alpha1ServingRuntimeSpec.

        ImagePullSecrets is an optional list of references to secrets in the same namespace to use for pulling any of the images used by this PodSpec. If specified, these secrets will be passed to individual puller implementations for them to use. For example, in the case of docker, only DockerConfig type secrets are honored. More info: https://kubernetes.io/docs/concepts/containers/images#specifying-imagepullsecrets-on-a-pod  # noqa: E501

        :param image_pull_secrets: The image_pull_secrets of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: list[V1LocalObjectReference]
        """

        self._image_pull_secrets = image_pull_secrets

    @property
    def labels(self):
        """Gets the labels of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Labels that will be add to the pod. More info: http://kubernetes.io/docs/user-guide/labels  # noqa: E501

        :return: The labels of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._labels

    @labels.setter
    def labels(self, labels):
        """Sets the labels of this V1alpha1ServingRuntimeSpec.

        Labels that will be add to the pod. More info: http://kubernetes.io/docs/user-guide/labels  # noqa: E501

        :param labels: The labels of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._labels = labels

    @property
    def multi_model(self):
        """Gets the multi_model of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Whether this ServingRuntime is intended for multi-model usage or not.  # noqa: E501

        :return: The multi_model of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: bool
        """
        return self._multi_model

    @multi_model.setter
    def multi_model(self, multi_model):
        """Sets the multi_model of this V1alpha1ServingRuntimeSpec.

        Whether this ServingRuntime is intended for multi-model usage or not.  # noqa: E501

        :param multi_model: The multi_model of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: bool
        """

        self._multi_model = multi_model

    @property
    def node_selector(self):
        """Gets the node_selector of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        NodeSelector is a selector which must be true for the pod to fit on a node. Selector which must match a node's labels for the pod to be scheduled on that node. More info: https://kubernetes.io/docs/concepts/configuration/assign-pod-node/  # noqa: E501

        :return: The node_selector of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._node_selector

    @node_selector.setter
    def node_selector(self, node_selector):
        """Sets the node_selector of this V1alpha1ServingRuntimeSpec.

        NodeSelector is a selector which must be true for the pod to fit on a node. Selector which must match a node's labels for the pod to be scheduled on that node. More info: https://kubernetes.io/docs/concepts/configuration/assign-pod-node/  # noqa: E501

        :param node_selector: The node_selector of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: dict(str, str)
        """

        self._node_selector = node_selector

    @property
    def protocol_versions(self):
        """Gets the protocol_versions of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Supported protocol versions (i.e. v1 or v2 or grpc-v1 or grpc-v2)  # noqa: E501

        :return: The protocol_versions of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: list[str]
        """
        return self._protocol_versions

    @protocol_versions.setter
    def protocol_versions(self, protocol_versions):
        """Sets the protocol_versions of this V1alpha1ServingRuntimeSpec.

        Supported protocol versions (i.e. v1 or v2 or grpc-v1 or grpc-v2)  # noqa: E501

        :param protocol_versions: The protocol_versions of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: list[str]
        """

        self._protocol_versions = protocol_versions

    @property
    def replicas(self):
        """Gets the replicas of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Configure the number of replicas in the Deployment generated by this ServingRuntime If specified, this overrides the podsPerRuntime configuration value  # noqa: E501

        :return: The replicas of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: int
        """
        return self._replicas

    @replicas.setter
    def replicas(self, replicas):
        """Sets the replicas of this V1alpha1ServingRuntimeSpec.

        Configure the number of replicas in the Deployment generated by this ServingRuntime If specified, this overrides the podsPerRuntime configuration value  # noqa: E501

        :param replicas: The replicas of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: int
        """

        self._replicas = replicas

    @property
    def storage_helper(self):
        """Gets the storage_helper of this V1alpha1ServingRuntimeSpec.  # noqa: E501


        :return: The storage_helper of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: V1alpha1StorageHelper
        """
        return self._storage_helper

    @storage_helper.setter
    def storage_helper(self, storage_helper):
        """Sets the storage_helper of this V1alpha1ServingRuntimeSpec.


        :param storage_helper: The storage_helper of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: V1alpha1StorageHelper
        """

        self._storage_helper = storage_helper

    @property
    def supported_model_formats(self):
        """Gets the supported_model_formats of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        Model formats and version supported by this runtime  # noqa: E501

        :return: The supported_model_formats of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: list[V1alpha1SupportedModelFormat]
        """
        return self._supported_model_formats

    @supported_model_formats.setter
    def supported_model_formats(self, supported_model_formats):
        """Sets the supported_model_formats of this V1alpha1ServingRuntimeSpec.

        Model formats and version supported by this runtime  # noqa: E501

        :param supported_model_formats: The supported_model_formats of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: list[V1alpha1SupportedModelFormat]
        """

        self._supported_model_formats = supported_model_formats

    @property
    def tolerations(self):
        """Gets the tolerations of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        If specified, the pod's tolerations.  # noqa: E501

        :return: The tolerations of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: list[V1Toleration]
        """
        return self._tolerations

    @tolerations.setter
    def tolerations(self, tolerations):
        """Sets the tolerations of this V1alpha1ServingRuntimeSpec.

        If specified, the pod's tolerations.  # noqa: E501

        :param tolerations: The tolerations of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: list[V1Toleration]
        """

        self._tolerations = tolerations

    @property
    def volumes(self):
        """Gets the volumes of this V1alpha1ServingRuntimeSpec.  # noqa: E501

        List of volumes that can be mounted by containers belonging to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes  # noqa: E501

        :return: The volumes of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: list[V1Volume]
        """
        return self._volumes

    @volumes.setter
    def volumes(self, volumes):
        """Sets the volumes of this V1alpha1ServingRuntimeSpec.

        List of volumes that can be mounted by containers belonging to the pod. More info: https://kubernetes.io/docs/concepts/storage/volumes  # noqa: E501

        :param volumes: The volumes of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: list[V1Volume]
        """

        self._volumes = volumes

    @property
    def worker_spec(self):
        """Gets the worker_spec of this V1alpha1ServingRuntimeSpec.  # noqa: E501


        :return: The worker_spec of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :rtype: V1alpha1WorkerSpec
        """
        return self._worker_spec

    @worker_spec.setter
    def worker_spec(self, worker_spec):
        """Sets the worker_spec of this V1alpha1ServingRuntimeSpec.


        :param worker_spec: The worker_spec of this V1alpha1ServingRuntimeSpec.  # noqa: E501
        :type: V1alpha1WorkerSpec
        """

        self._worker_spec = worker_spec

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1alpha1ServingRuntimeSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1alpha1ServingRuntimeSpec):
            return True

        return self.to_dict() != other.to_dict()



# Copyright 2023 The KServe Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    KServe

    Python SDK for KServe  # noqa: E501

    The version of the OpenAPI document: v0.1
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from kserve.configuration import Configuration


class V1beta1ComponentStatusSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'address': 'KnativeAddressable',
        'grpc_url': 'KnativeURL',
        'latest_created_revision': 'str',
        'latest_ready_revision': 'str',
        'latest_rolledout_revision': 'str',
        'previous_rolledout_revision': 'str',
        'rest_url': 'KnativeURL',
        'traffic': 'list[KnativeDevServingPkgApisServingV1TrafficTarget]',
        'url': 'KnativeURL'
    }

    attribute_map = {
        'address': 'address',
        'grpc_url': 'grpcUrl',
        'latest_created_revision': 'latestCreatedRevision',
        'latest_ready_revision': 'latestReadyRevision',
        'latest_rolledout_revision': 'latestRolledoutRevision',
        'previous_rolledout_revision': 'previousRolledoutRevision',
        'rest_url': 'restUrl',
        'traffic': 'traffic',
        'url': 'url'
    }

    def __init__(self, address=None, grpc_url=None, latest_created_revision=None, latest_ready_revision=None, latest_rolledout_revision=None, previous_rolledout_revision=None, rest_url=None, traffic=None, url=None, local_vars_configuration=None):  # noqa: E501
        """V1beta1ComponentStatusSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._address = None
        self._grpc_url = None
        self._latest_created_revision = None
        self._latest_ready_revision = None
        self._latest_rolledout_revision = None
        self._previous_rolledout_revision = None
        self._rest_url = None
        self._traffic = None
        self._url = None
        self.discriminator = None

        if address is not None:
            self.address = address
        if grpc_url is not None:
            self.grpc_url = grpc_url
        if latest_created_revision is not None:
            self.latest_created_revision = latest_created_revision
        if latest_ready_revision is not None:
            self.latest_ready_revision = latest_ready_revision
        if latest_rolledout_revision is not None:
            self.latest_rolledout_revision = latest_rolledout_revision
        if previous_rolledout_revision is not None:
            self.previous_rolledout_revision = previous_rolledout_revision
        if rest_url is not None:
            self.rest_url = rest_url
        if traffic is not None:
            self.traffic = traffic
        if url is not None:
            self.url = url

    @property
    def address(self):
        """Gets the address of this V1beta1ComponentStatusSpec.  # noqa: E501


        :return: The address of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: KnativeAddressable
        """
        return self._address

    @address.setter
    def address(self, address):
        """Sets the address of this V1beta1ComponentStatusSpec.


        :param address: The address of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: KnativeAddressable
        """

        self._address = address

    @property
    def grpc_url(self):
        """Gets the grpc_url of this V1beta1ComponentStatusSpec.  # noqa: E501


        :return: The grpc_url of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: KnativeURL
        """
        return self._grpc_url

    @grpc_url.setter
    def grpc_url(self, grpc_url):
        """Sets the grpc_url of this V1beta1ComponentStatusSpec.


        :param grpc_url: The grpc_url of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: KnativeURL
        """

        self._grpc_url = grpc_url

    @property
    def latest_created_revision(self):
        """Gets the latest_created_revision of this V1beta1ComponentStatusSpec.  # noqa: E501

        Latest revision name that is created  # noqa: E501

        :return: The latest_created_revision of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: str
        """
        return self._latest_created_revision

    @latest_created_revision.setter
    def latest_created_revision(self, latest_created_revision):
        """Sets the latest_created_revision of this V1beta1ComponentStatusSpec.

        Latest revision name that is created  # noqa: E501

        :param latest_created_revision: The latest_created_revision of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: str
        """

        self._latest_created_revision = latest_created_revision

    @property
    def latest_ready_revision(self):
        """Gets the latest_ready_revision of this V1beta1ComponentStatusSpec.  # noqa: E501

        Latest revision name that is in ready state  # noqa: E501

        :return: The latest_ready_revision of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: str
        """
        return self._latest_ready_revision

    @latest_ready_revision.setter
    def latest_ready_revision(self, latest_ready_revision):
        """Sets the latest_ready_revision of this V1beta1ComponentStatusSpec.

        Latest revision name that is in ready state  # noqa: E501

        :param latest_ready_revision: The latest_ready_revision of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: str
        """

        self._latest_ready_revision = latest_ready_revision

    @property
    def latest_rolledout_revision(self):
        """Gets the latest_rolledout_revision of this V1beta1ComponentStatusSpec.  # noqa: E501

        Latest revision name that is rolled out with 100 percent traffic  # noqa: E501

        :return: The latest_rolledout_revision of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: str
        """
        return self._latest_rolledout_revision

    @latest_rolledout_revision.setter
    def latest_rolledout_revision(self, latest_rolledout_revision):
        """Sets the latest_rolledout_revision of this V1beta1ComponentStatusSpec.

        Latest revision name that is rolled out with 100 percent traffic  # noqa: E501

        :param latest_rolledout_revision: The latest_rolledout_revision of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: str
        """

        self._latest_rolledout_revision = latest_rolledout_revision

    @property
    def previous_rolledout_revision(self):
        """Gets the previous_rolledout_revision of this V1beta1ComponentStatusSpec.  # noqa: E501

        Previous revision name that is rolled out with 100 percent traffic  # noqa: E501

        :return: The previous_rolledout_revision of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: str
        """
        return self._previous_rolledout_revision

    @previous_rolledout_revision.setter
    def previous_rolledout_revision(self, previous_rolledout_revision):
        """Sets the previous_rolledout_revision of this V1beta1ComponentStatusSpec.

        Previous revision name that is rolled out with 100 percent traffic  # noqa: E501

        :param previous_rolledout_revision: The previous_rolledout_revision of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: str
        """

        self._previous_rolledout_revision = previous_rolledout_revision

    @property
    def rest_url(self):
        """Gets the rest_url of this V1beta1ComponentStatusSpec.  # noqa: E501


        :return: The rest_url of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: KnativeURL
        """
        return self._rest_url

    @rest_url.setter
    def rest_url(self, rest_url):
        """Sets the rest_url of this V1beta1ComponentStatusSpec.


        :param rest_url: The rest_url of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: KnativeURL
        """

        self._rest_url = rest_url

    @property
    def traffic(self):
        """Gets the traffic of this V1beta1ComponentStatusSpec.  # noqa: E501

        Traffic holds the configured traffic distribution for latest ready revision and previous rolled out revision.  # noqa: E501

        :return: The traffic of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: list[KnativeDevServingPkgApisServingV1TrafficTarget]
        """
        return self._traffic

    @traffic.setter
    def traffic(self, traffic):
        """Sets the traffic of this V1beta1ComponentStatusSpec.

        Traffic holds the configured traffic distribution for latest ready revision and previous rolled out revision.  # noqa: E501

        :param traffic: The traffic of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: list[KnativeDevServingPkgApisServingV1TrafficTarget]
        """

        self._traffic = traffic

    @property
    def url(self):
        """Gets the url of this V1beta1ComponentStatusSpec.  # noqa: E501


        :return: The url of this V1beta1ComponentStatusSpec.  # noqa: E501
        :rtype: KnativeURL
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this V1beta1ComponentStatusSpec.


        :param url: The url of this V1beta1ComponentStatusSpec.  # noqa: E501
        :type: KnativeURL
        """

        self._url = url

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1beta1ComponentStatusSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1beta1ComponentStatusSpec):
            return True

        return self.to_dict() != other.to_dict()
